/* MAC 440/5759 - Sistemas de Objetos Distribudos  */
/* IME-USP - 1o.sem/2002 - Prof. Fabio Kon          */
/*                                                  */
/* Projeto Final: Siscoma                           */
/*                                                  */
/* Emerson Luiz Navarro  - emr@linux.ime.usp.br     */
/* Leo Kazuhiro Ueda     -  lku@ime.usp.br          */
/* Pratip Roy Chowdhury  - roy@linux.ime.usp.br     */
/* Ricardo Hideo Sahara  - hideo@linux.ime.usp.br   */
/*                                                  */


/* Siscoma.java */

import siscoma.orb.*;
import siscoma.orb.ServentePackage.*;
import java.util.*;
import java.io.*;

import org.omg.CosNaming.*;

public class Siscoma {
//public class PrimeiroServente {

  // Parmetros:
  public static int TAMBLOCO = 200; //  2048;
  public static String DIRCOMP = "./";
  public static String DIRGRAV = "./grav";
  public static int TDV = 5;
  
  private static org.omg.CORBA.ORB Orb;
  private static org.omg.PortableServer.POA Poa;

  static String ArqIor;
  static String ArqAmigos;
  static String Nome;
  static String NomeSN;
  static String UrlSN;

  public static void main (String args[]) {

    processaArgs(args);

    if (UrlSN == null) 
      UrlSN = "file:/tmp/NS_Ref";

    try {
      java.util.Properties props = new java.util.Properties();
      props.setProperty("ORBInitRef.NameService", UrlSN);

      Orb = org.omg.CORBA.ORB.init(args, props);
      Poa = org.omg.PortableServer.POAHelper.narrow
	( Orb.resolve_initial_references("RootPOA"));
      Poa.the_POAManager().activate();

      Collection amigos = new HashSet();
      if (ArqAmigos != null) {
	leAmigos(ArqAmigos, amigos);
      }
      ServenteImpl si = new ServenteImpl(Orb, Poa, amigos, Nome,
					 ArqIor, NomeSN);

    }
    catch (Exception e) { e.printStackTrace(); }

    Orb.run();

  }


  public static void processaArgs(String [] args) {
    if (args.length < 2)
      erro("faltam parmetros.");

    ArqIor = ArqAmigos = Nome = NomeSN = null;

    Nome = args[0];

    int i = 1;
    while (i < args.length) {
      if (args[i].equals("-sn")) {
	i++;
	if (i >= args.length)
	  erro("falta a url do servio de nomes.");
	UrlSN = args[i];
	NomeSN = Nome;	
      }
      else if (args[i].equals("-am")) {
	i++;
	if (i >= args.length)
	  erro("falta nome do arquivo de amigos.");
	ArqAmigos = args[i];
      }
      else if (args[i].equals("-i")) {
	i++;
	if (i >= args.length)
	  erro("falta o nome do arquivo de sada da ior.");
	ArqIor = args[i];
      }
      else if (args[i].equals("-t")) {
	i++;
	if (i >= args.length)
	  erro("falta um parmetro.");
	TDV = Integer.parseInt(args[i]);
      }
      else if (args[i].equals("-b")) {
	i++;
	if (i >= args.length)
	  erro("falta um parmetro.");
	TAMBLOCO = Integer.parseInt(args[i]);
      }
      else if (args[i].equals("-dc")) {
	i++;
	if (i >= args.length)
	  erro("falta um parmetro.");
	DIRCOMP = args[i];
      }
      else if (args[i].equals("-dg")) {
	i++;
	if (i >= args.length)
	  erro("falta um parmetro.");
	DIRGRAV = args[i];
      }
      else {
	System.err.println("aviso: parmetro invlido: " + args[i]);
      }
      i++;
    }
  }
  
  public static void erro(String msg) {
    System.err.println("erro: " + msg);
    imprimeAjuda();
    System.exit(-1);
  }

  public static void imprimeAjuda() {
    System.err.println("uso: Siscoma <nome> ");
    System.err.println("             [-sn <url>] [-am <arq.amigos>] " +
		       "[-i <arq.ior>]");
    System.err.println("             [-t <tdv>] [-b <tam.bloco>] " +
		       " [-dc <dir.comp] [-dg <dir.grav>]");
    System.err.println();
    System.err.println("Propriedades: ");
    System.err.println("<nome>             um identificador para o programa");
    System.err.println("-dc <dir.comp>     diretrio compartilhado");
    System.err.println("-dc <dir.grav>     diretrio onde sero gravados os " +
		       " arquivos recebidos");
    System.err.println("-t <tdv>           tempo de vida das mensagens");
    System.err.println("-b <tam.bloco>     tamanho do bloco enviado/recebido " +
		       "em cada chamada de " );
    System.err.println("                   pedido de arquivo");
    System.err.println();
    System.err.println("Opes de conexo:");
    System.err.println("-sn <url>          usa o servio de nomes cuja IOR est " +
		       "localizada em <url>");
    System.err.println("-am <arq.amigos>   l IORs de <arq.amigos>, onde " +
		       "cada linha  uma string que ");
    System.err.println("                   representa uma IOR");
    System.err.println("-i <arq.ior>       grava a prpria IOR em <arq.ior>");
    System.err.println();
    System.err.println("Alm de <nome>,  necessrio especificar pelo menos uma " +
		       "opo de conexo.");
  }



  public static void leAmigos(String arqAmigos, Collection amigos) {
    try {
      BufferedReader br = new BufferedReader(new FileReader(arqAmigos));
      
      String linha = br.readLine();
      while (linha  != null) {
	System.out.println(">Ior: "+ linha +"\n");
	Servente s = ServenteHelper.narrow(Orb.string_to_object(linha));
	amigos.add(s);
	linha = br.readLine();
      }
    }
    catch (Exception e) { e.printStackTrace(); System.exit(-10); }
  }

}


